@@ -5,8 +5,12 @@ import sqlparse
 from sqlparse.sql import Comparison, Identifier, Token, Where
 
 from mycli.packages.parseutils import extract_tables, find_prev_keyword, last_word
+from mycli.packages.special.main import COMMANDS as SPECIAL_COMMANDS
 from mycli.packages.special.main import parse_special_command
 
+sqlparse.engine.grouping.MAX_GROUPING_DEPTH = None  # type: ignore[assignment]
+sqlparse.engine.grouping.MAX_GROUPING_TOKENS = None  # type: ignore[assignment]
+
 _ENUM_VALUE_RE = re.compile(
     r"(?P<lhs>(?:`[^`]+`|[\w$]+)(?:\.(?:`[^`]+`|[\w$]+))?)\s*=\s*$",
     re.IGNORECASE,
@@ -123,8 +127,12 @@ def suggest_type(full_text: str, text_before_cursor: str) -> list[dict[str, Any]
         # Be careful here because trivial whitespace is parsed as a statement,
         # but the statement won't have a first token
         tok1 = statement.token_first()
-        if tok1 and (tok1.value == "source" or tok1.value.startswith("\\")):
+        # lenient because \. will parse as two tokens
+        if tok1 and tok1.value.startswith('\\'):
             return suggest_special(text_before_cursor)
+        elif tok1:
+            if tok1.value.lower() in SPECIAL_COMMANDS:
+                return suggest_special(text_before_cursor)
 
     last_token = statement and statement.token_prev(len(statement.tokens))[1] or ""
 
@@ -143,7 +151,13 @@ def suggest_special(text: str) -> list[dict[str, Any]]:
     if cmd in ("\\u", "\\r"):
         return [{"type": "database"}]
 
-    if cmd in ("\\T"):
+    if cmd.lower() in ('use', 'connect'):
+        return [{'type': 'database'}]
+
+    if cmd in (r'\T', r'\Tr'):
+        return [{"type": "table_format"}]
+
+    if cmd.lower() in ('tableformat', 'redirectformat'):
         return [{"type": "table_format"}]
 
     if cmd in ["\\f", "\\fs", "\\fd"]:
@@ -155,7 +169,7 @@ def suggest_special(text: str) -> list[dict[str, Any]]:
             {"type": "view", "schema": []},
             {"type": "schema"},
         ]
-    elif cmd in ["\\.", "source"]:
+    elif cmd.lower() in ["\\.", "source"]:
         return [{"type": "file_name"}]
     if cmd in ["\\llm", "\\ai"]:
         return [{"type": "llm"}]
@@ -251,6 +265,8 @@ def suggest_based_on_last_token(
 
         # We're probably in a function argument list
         return [{"type": "column", "tables": extract_tables(full_text)}]
+    elif token_v in ("call"):
+        return [{"type": "procedure", "schema": []}]
     elif token_v in ("set", "order by", "distinct"):
         return [{"type": "column", "tables": extract_tables(full_text)}]
     elif token_v == "as":
@@ -287,8 +303,12 @@ def suggest_based_on_last_token(
                 {"type": "alias", "aliases": aliases},
                 {"type": "keyword"},
             ]
-    elif (token_v.endswith("join") and isinstance(token, Token) and token.is_keyword) or (
-        token_v in ("copy", "from", "update", "into", "describe", "truncate", "desc", "explain")
+    elif (
+        (token_v.endswith("join") and isinstance(token, Token) and token.is_keyword)
+        or (token_v in ("copy", "from", "update", "into", "describe", "truncate", "desc", "explain"))
+        # todo: the create table regex fails to match on multi-statement queries, which
+        # suggests a bug above in suggest_type()
+        or (token_v == "like" and re.match(r'^\s*create\s+table\s', full_text, re.IGNORECASE))
     ):
         schema = (identifier and identifier.get_parent_name()) or []
 
@@ -298,7 +318,7 @@ def suggest_based_on_last_token(
 
         if not schema:
             # Suggest schemas
-            suggest.insert(0, {"type": "schema"})
+            suggest.append({"type": "database"})
 
         # Only tables can be TRUNCATED, otherwise suggest views
         if token_v != "truncate":
@@ -335,17 +355,17 @@ def suggest_based_on_last_token(
 
             # The lists of 'aliases' could be empty if we're trying to complete
             # a GRANT query. eg: GRANT SELECT, INSERT ON <tab>
-            # In that case we just suggest all tables.
+            # In that case we just suggest all schemata and all tables.
             if not aliases:
+                suggest.append({"type": "database"})
                 suggest.append({"type": "table", "schema": parent})
             return suggest
 
-    elif token_v in ("use", "database", "template", "connect"):
+    elif token_v in ("database", "template"):
         # "\c <db", "use <db>", "DROP DATABASE <db>",
         # "CREATE DATABASE <newdb> WITH TEMPLATE <db>"
         return [{"type": "database"}]
-    elif token_v == "tableformat":
-        return [{"type": "table_format"}]
+
     elif token_v.endswith(",") or is_operand(token_v) or token_v in ["=", "and", "or"]:
         original_text = text_before_cursor
         prev_keyword, text_before_cursor = find_prev_keyword(text_before_cursor)
