@@ -1,13 +1,16 @@
 from __future__ import annotations
 
 import re
-from typing import Generator
+from typing import Any, Generator
 
 import sqlglot
 import sqlparse
 from sqlparse.sql import Function, Identifier, IdentifierList, Token, TokenList
 from sqlparse.tokens import DML, Keyword, Punctuation
 
+sqlparse.engine.grouping.MAX_GROUPING_DEPTH = None  # type: ignore[assignment]
+sqlparse.engine.grouping.MAX_GROUPING_TOKENS = None  # type: ignore[assignment]
+
 cleanup_regex: dict[str, re.Pattern] = {
     # This matches only alphanumerics and underscores.
     "alphanum_underscore": re.compile(r"(\w+)$"),
@@ -20,6 +23,17 @@ cleanup_regex: dict[str, re.Pattern] = {
 }
 
 
+def is_valid_connection_scheme(text: str) -> tuple[bool, str | None]:
+    # exit early if the text does not resemble a DSN URI
+    if "://" not in text:
+        return False, None
+    scheme = text.split("://")[0]
+    if scheme not in ("mysql", "mysqlx", "tcp", "socket", "ssh"):
+        return False, scheme
+    else:
+        return True, None
+
+
 def last_word(text: str, include: str = "alphanum_underscore") -> str:
     r"""
     Find the last word in a sentence.
@@ -77,7 +91,7 @@ def is_subselect(parsed: TokenList) -> bool:
     return False
 
 
-def extract_from_part(parsed: TokenList, stop_at_punctuation: bool = True) -> Generator[str, None, None]:
+def extract_from_part(parsed: TokenList, stop_at_punctuation: bool = True) -> Generator[Any, None, None]:
     tbl_prefix_seen = False
     for item in parsed.tokens:
         if tbl_prefix_seen:
@@ -123,7 +137,7 @@ def extract_from_part(parsed: TokenList, stop_at_punctuation: bool = True) -> Ge
                     break
 
 
-def extract_table_identifiers(token_stream: TokenList) -> Generator[tuple[str | None, str, str], None, None]:
+def extract_table_identifiers(token_stream: Generator[Any, None, None]) -> Generator[tuple[str | None, str, str], None, None]:
     """yields tuples of (schema_name, table_name, table_alias)"""
 
     for item in token_stream:
@@ -187,15 +201,15 @@ def extract_tables_from_complete_statements(sql: str) -> list[tuple[str | None,
         return []
 
     finely_parsed = []
-    for statement in roughly_parsed:
+    for rough_statement in roughly_parsed:
         try:
-            finely_parsed.append(sqlglot.parse_one(str(statement), read='mysql'))
+            finely_parsed.append(sqlglot.parse_one(str(rough_statement), read='mysql'))
         except sqlglot.errors.ParseError:
             pass
 
     tables = []
-    for statement in finely_parsed:
-        for identifier in statement.find_all(sqlglot.exp.Table):
+    for fine_statement in finely_parsed:
+        for identifier in fine_statement.find_all(sqlglot.exp.Table):
             if identifier.parent_select and identifier.parent_select.sql().startswith('WITH'):
                 continue
             tables.append((
@@ -261,26 +275,49 @@ def query_has_where_clause(query: str) -> bool:
     return any(isinstance(token, sqlparse.sql.Where) for token_list in sqlparse.parse(query) for token in token_list)
 
 
-def is_destructive(queries: str) -> bool:
-    """Returns if any of the queries in *queries* is destructive."""
-    keywords = ("drop", "shutdown", "delete", "truncate", "alter")
+# todo: handle "UPDATE LOW_PRIORITY" and "UPDATE IGNORE"
+def query_is_single_table_update(query: str) -> bool:
+    """Check if a query is a simple single-table UPDATE."""
+    cleaned_query = sqlparse.format(query, strip_comments=True)
+    if not cleaned_query:
+        return False
+    parsed = sqlparse.parse(cleaned_query)
+    if not parsed:
+        return False
+    statement = parsed[0]
+    return (
+        statement[0].value.lower() == 'update'
+        and statement[1].is_whitespace
+        and ',' not in statement[2].value  # multiple tables
+        and statement[3].is_whitespace
+        and statement[4].value.lower() == 'set'
+    )
+
+
+def is_destructive(keywords: list[str], queries: str) -> bool:
+    """Returns True if any of the queries in *queries* is destructive."""
     for query in sqlparse.split(queries):
-        if query:
-            if query_starts_with(query, list(keywords)) is True:
-                return True
-            elif query_starts_with(query, ["update"]) is True and not query_has_where_clause(query):
+        if not query:
+            continue
+        # subtle: if "UPDATE" is one of our keywords AND "query" starts with "UPDATE"
+        if query_starts_with(query, keywords) and query_starts_with(query, ["update"]):
+            if query_has_where_clause(query) and query_is_single_table_update(query):
+                return False
+            else:
                 return True
+        if query_starts_with(query, keywords):
+            return True
 
     return False
 
 
-def is_dropping_database(queries: list[str], dbname: str | None) -> bool:
+def is_dropping_database(queries: str, dbname: str | None) -> bool:
     """Determine if the query is dropping a specific database."""
     result = False
     if dbname is None:
         return False
 
-    def normalize_db_name(db):
+    def normalize_db_name(db: str) -> str:
         return db.lower().strip('`"')
 
     dbname = normalize_db_name(dbname)
